module ToSpotlight
  class SpotlightTransferJob < ActiveJob::Base
    queue_as :default

    def perform(works, mappings, transfer)
      base_url    = ::Account.where(tenant: Apartment::Tenant.current).first.cname
      csv_headers = %w[url full_title_tesim] + mappings.keys + %w[children]
      csv_array   = [csv_headers.join(',')]
      #items = []
      works.each do |wk|
        #if comp
        #  items << wk.id if wk.file_set_ids.length>1
        #  next unless wk.file_set_ids.length>1
        #else
        #  items << wk.id unless wk.file_set_ids.length>1
        #  next if wk.file_set_ids.length>1
        #end
        #index += 1
        comp     = false #wk.file_set_ids.length > 1
        children = []
        if comp
          index = wk.ordered_file_set_ids.length * -1
          wk.ordered_file_set_ids.each do |f_id|
            csv_array << empty_fs_line(f_id, csv_headers.length - 1)
            children << index
            index += 1
          end
        end

        doc = ::SolrDocument.find wk.id

        line_hash                     = {}
        line_hash['url']              = "http://#{base_url}/concern/#{underscore_name(wk)}/#{wk.id}/manifest.json" #url
        line_hash['full_title_tesim'] = (doc.title.length > 1 ? doc.title.join('; ') : doc.title.first) #full_title_tesim

        mappings.each do |skey, field|
          line_hash[skey] = create_cell doc, field
        end

        line_hash['children'] = children.join('|')

        csv_array << line_hash.values_at(*csv_headers).map { |cell| cell = '' if cell.nil?; "\"#{cell.gsub("\"", "\"\"")}\"" }.join(',')

      end

      dir = Rails.root.join('public', 'uploads', 'csvs')
      Dir.mkdir(dir) unless Dir.exist?(dir)
      time     = DateTime.now.strftime('%s')
      filename = "spotlight_export-#{time}.csv"
      File.open(dir.join(filename), 'wb') do |file|
        file.write(csv_array.join("\n"))
      end
      res = post_response URI("#{transfer.spotlight_url}/from_hyrax/receive"),
                          csv_url:  "https://#{base_url}/uploads/csvs/#{filename}",
                          transfer: transfer.attributes.slice(:user, :exhibit), #only allow user+exhibit
                          token: 'secret_token'

    end

    def post_response uri, options = {}
      http             = Net::HTTP.new(uri.host, uri.port)
      http.use_ssl     = true
      http.verify_mode = OpenSSL::SSL::VERIFY_NONE
      request          = Net::HTTP::Post.new(uri.request_uri, 'Content-Type' => 'application/json')
      request.body     = options.to_json
      http.request(request)
    end

    def base_url
      @base_url ||= ::Account.where(tenant: Apartment::Tenant.current).first.cname
    end

    def empty_fs_line fs_id, commas
      "http://#{base_url}/downloads/#{fs_id}?locale=en" + (',' * commas)
    end

    def create_cell w, field
      if w.respond_to?((field + '_label').to_sym)
        w.send(field + '_label').join(';')
      elsif w.respond_to?(field.to_sym)
        if w.send(field).is_a?(Array)
          w.send(field).join(';')
        else
          w.send(field)
        end
      end
    end

    def underscore_name work
      work.class.to_s.underscore + 's'
    end

    def get_location list
      output = []
      list.each do |loc|
        output += loc.rdf_label || []
      end
      output.join('; ')
    end

    def get_resource list
      output = []
      list.each do |url|
        output << if url.include? 'http'
                    Hyrax::ResourceTypesService.label(url)
                  else
                    url
                  end
      end
      output.join('; ')
    end

    def get_rights list
      output = []
      list.each do |url|
        output << if url.include? 'http'
                    ::VaultRightsStatementsService.label(url)
                  else
                    url
                  end
      end
      #output = list.map { |url| url.include? "http" ? IaffRightsStatementService.label(url) : url }
      output.join('; ')
    end

    #def csv_headers
    #  "url,full_title_tesim,spotlight_upload_dc_description_tesim,spotlight_upload_dc_Subject_tesim,spotlight_upload_dc_Subjects_tesim,spotlight_upload_dc_Subjects_ftesim,spotlight_upload_Creator_tesim,spotlight_upload_Publisher_tesim,spotlight_upload_Contributors_tesim,spotlight_upload_dc_Date_tesi,spotlight_upload_dc_Date-Created_Searchable_ftesi,spotlight_upload_dc_Date-Created_Searchable_tesi,spotlight_upload_dc_Type_Genre_tesim,spotlight_upload_dc_Type_Genre_ftesim,spotlight_upload_Format_tesim,spotlight_upload_Identifier_tesim,spotlight_upload_Source_tesim,spotlight_upload_Language_tesim,spotlight_upload_Language_ftesim,spotlight_upload_Relation_tesim,spotlight_upload_Coverage_tesim,spotlight_upload_Rights_tesim,spotlight_upload_Provenance_tesim,spotlight_upload_dc_Title_Alternative_tesim,spotlight_upload_Description-Table-Of-Contents_tesim,spotlight_upload_Description-Abstract_tesim,spotlight_upload_Format-Extent_tesim,spotlight_upload_Format-Medium_tesim,spotlight_upload_Identifier-Bibliographic-Citation_tesim,spotlight_upload_dc_Relation_IsPartOf_Collection_tesim,spotlight_upload_dc_Relation_IsPartOf_Collection_ftesim,spotlight_upload_dc_Coverage-Spatial_Location_tesim,spotlight_upload_dc_Coverage-Spatial_Location_ftesim,spotlight_upload_Coverage-Temporal_tesim,spotlight_upload_Date-Digitized_tesi,spotlight_upload_dc_Description_Transcript_tesim,spotlight_upload_dc_Subject_People_tesim,spotlight_upload_dc_Subject_People_ftesim,spotlight_upload_Sketchfab-uid_tesim,spotlight_annotation_x_dbsm,spotlight_annotation_y_dbsm,spotlight_annotation_width_dbsm,spotlight_annotation_height_dbsm,spotlight_annotation_title_tesim,spotlight_annotation_author_tesim,spotlight_annotation_description_tesim,spotlight_annotation_date_tesim,spotlight_annotation_publisher_tesim,spotlight_annotation_publisher-place_tesim,spotlight_annotation_publisher-date_tesim,spotlight_annotation_people_tesim,spotlight_annotation_locations_tesim,spotlight_annotation_transcript_tesim,spotlight_annotation_genre_tesim,spotlight_annotation_public_isim,spotlight_upload_parent_tesim,spotlight_upload_Commentary_tesim,spotlight_upload_UserDefined_tesim,children"
    #end

  end
end
